import 'dart:async';
import 'dart:ffi';

import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'dart:convert';

class MyTransporterWebView extends ModalRoute {
  final Completer<WebViewController> _controller =
      Completer<WebViewController>();

  final String publicKey;

  final String email;

  final String firstName;

  final String lastName;

  final Function onCancel;

  final Function onTransport;

  final double pickupLatitude;

  final String pickupLocation;

  final double pickupLongitude;

  final String phone;

  final Function onError;

  MyTransporterWebView(
      {required this.publicKey,
      required this.email,
      required this.firstName,
      required this.lastName,
      required this.pickupLatitude,
      required this.pickupLongitude,
      required this.pickupLocation,
      required this.phone,
      required this.onCancel,
      required this.onTransport,
      required this.onError});

  @override
  Widget buildPage(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
  ) {
    return Material(
      type: MaterialType.transparency,
      child: SafeArea(
        child: WebView(
          initialUrl: Uri.dataFromString(
                  '<html lang="en"><head> \n\n' +
                      '                        <meta charset="UTF-8">\n' +
                      '                        <meta http-equiv="X-UA-Compatible" content="ie=edge">\n' +
                      '                        <title>MyTransporter</title>\n' +
                      '                </head>\n' +
                      '                      <body  onload="initiateTransport()" style="background-color:#fff;height:100vh ">\n' +
                      '                       <script src="https://lib.mytransporter.io/js/mtp.js"></script>\n' +
                      '                        \n' +
                      '                          <script type="text/javascript">\n' +
                      '                                window.onload = initiateTransport;\n' +
                      '                                function initiateTransport(){\n' +
                      '\n' +
                      '                                    var transporter =  MyTransporterSDK.transport({\n' +
                      '                                        public_key: "$publicKey",\n' +
                      '                                        first_name: "$firstName}",\n' +
                      '                                        last_name: "$lastName}",\n' +
                      '                                        email: "$email",\n' +
                      '                                        phone: "$phone",\n' +
                      '                                        pickup_latitude: "$pickupLatitude",\n' +
                      '                                        pickup_longitude: "$pickupLongitude",\n' +
                      '                                        pickup_location: "$pickupLocation",\n' +
                      '                                        callback: function (response) {\n' +
                      '                                           console.log("callback Response", response); \n' +
                      '                                           if(response.status =="success"){\n' +
                      '                                            var response_data = {event:"transport_initiated", data:response};\n' +
                      '                                            Print.postMessage(JSON.stringify(response_data))\n' +
                      '                                           }\n' +
                      '                                           else if(response.code == "E01"){\n' +
                      '                                            var response_data = {event:"closed"};\n' +
                      '                                            Print.postMessage(JSON.stringify(response_data))\n' +
                      '                                           }\n' +
                      '                                           else{\n' +
                      '                                            var response_data = {event:"error",message:response.message};\n' +
                      '                                            Print.postMessage(JSON.stringify(response_data))\n' +
                      '                                           }\n' +
                      '                                      },\n' +
                      '                                    })\n' +
                      '                                }\n' +
                      ' \n' +
                      '                        </script> \n' +
                      '                </body>\n' +
                      '        </html> ',
                  mimeType: 'text/html')
              .toString(),
          javascriptMode: JavascriptMode.unrestricted,
          javascriptChannels: Set.from([
            JavascriptChannel(
                name: 'Print',
                onMessageReceived: (JavascriptMessage message) {
                  //This is where you receive message from
                  var response = json.decode(message.message);
                  print(response);
                  switch (response["event"]) {
                    case "closed":
                      onCancel({"status": "closed"});
                      Navigator.pop(context);
                      break;
                    case "error":
                      onError({"status": "error", "message": response.message});
                      Navigator.pop(context);
                      break;
                    case "verified":
                      onTransport({
                        "status": "success",
                        "data": response,
                      });
                      Navigator.pop(context);
                      break;
                    default:
                      break;
                  }
                })
          ]),
          onWebViewCreated: (WebViewController webViewController) {
            _controller.complete(webViewController);
          },
          onPageStarted: (String url) {},
          onPageFinished: (String url) {},
          gestureNavigationEnabled: true,
        ),
      ),
    );
  }

  @override
  bool get opaque => false;

  @override
  bool get barrierDismissible => true;

  @override
  Color get barrierColor => Colors.black.withOpacity(0.5);

  @override
  String get barrierLabel => "test";

  @override
  bool get maintainState => true;

  @override
  Duration get transitionDuration => Duration(milliseconds: 500);
}
